@@ -1,20 +1,15 @@
 #!/usr/bin/env bash
 
-set -e
-set -u
-
 ## START STANDARD BUILD SCRIPT INCLUDE
 # adjust relative paths as necessary
 THIS_SCRIPT="$(readlink -f "${BASH_SOURCE[0]}")"
-. "${THIS_SCRIPT%/*}/../resources/build/build-utils.sh"
+. "${THIS_SCRIPT%/*}/../resources/build/builder.inc.sh"
 
 ## END STANDARD BUILD SCRIPT INCLUDE
 
 . "$KEYMAN_ROOT/resources/shellHelperFunctions.sh"
 . "$THIS_SCRIPT_PATH/commands.inc.sh"
 
-cd "$THIS_SCRIPT_PATH"
-
 ################################ Main script ################################
 
 cleanup_visual_studio_path
