@@ -21,6 +21,7 @@
  *
  */
 
+#include <errno.h>
 #include <ibus.h>
 #include <string.h>
 #include <stdio.h>
@@ -38,29 +39,21 @@
 #include "engine.h"
 #include "keycodes.h"
 
-// Fallback for older ibus versions that don't define IBUS_PREFILTER_MASK
-#ifndef IBUS_HAS_PREFILTER
-#ifdef KEYMAN_PKG_BUILD
-// When building packages on Ubuntu and Debian servers we probably don't have
-// a patched ibus available and additionally treat warnings as errors, but
-// still want to build packages.
-#pragma message "Compiling against ibus version that does not include prefilter mask patch\n(https://github.com/ibus/ibus/pull/2440). Output ordering guarantees will be disabled."
-#else
-#warning Compiling against ibus version that does not include prefilter mask patch (https://github.com/ibus/ibus/pull/2440). Output ordering guarantees will be disabled.
-#endif
-
-#define IBUS_PREFILTER_MASK (1 << 23)
-#endif
-
 #define MAXCONTEXT_ITEMS 128
+
+// Values from /usr/include/linux/input-event-codes.h
 #define KEYMAN_BACKSPACE 14
 #define KEYMAN_BACKSPACE_KEYSYM  IBUS_KEY_BackSpace
-#define KEYMAN_LCTRL 29 // 0x1D
-#define KEYMAN_LALT  56 // 0x38
-#define KEYMAN_RCTRL 97 // 0x61
-#define KEYMAN_RALT 100 // 0x64
-#define KEYMAN_F24_KEYCODE_OUTPUT_SENTINEL  202
-#define KEYMAN_NOCHAR_KEYSYM (0xfdd0 | 0x1000000) // Unicode NOCHAR
+#define KEYMAN_LCTRL  29 // 0x1D
+#define KEYMAN_LSHIFT 42 // 0x2A
+#define KEYMAN_RSHIFT 54 // 0x36
+#define KEYMAN_LALT   56 // 0x38
+#define KEYMAN_RCTRL  97 // 0x61
+#define KEYMAN_RALT  100 // 0x64
+
+// TODO: Move to a common header file. Also defined in tests/testfixture.h
+// and keyman-system-service/src/OrderedOutputDevice.cpp
+#define KEYMAN_F24_KEYCODE_OUTPUT_SENTINEL 194  // 0xC2
 
 typedef struct _IBusKeymanEngine IBusKeymanEngine;
 typedef struct _IBusKeymanEngineClass IBusKeymanEngineClass;
@@ -68,11 +61,12 @@ typedef struct _IBusKeymanEngineClass IBusKeymanEngineClass;
 #define MAX_QUEUE_SIZE 100
 
 typedef struct _commit_queue_item {
-  // char_buffer and emitting_keystroke as well as more than one queue
-  // item are only used if ibus supports prefilter but the client
-  // doesn't support surrounding text (non-compliant app)
+  // char_buffer, emitting_keystroke and code_points_to_delete as well as
+  // more than one queue item are only used if the client doesn't
+  // support surrounding text (non-compliant app)
   gchar *char_buffer;
   gboolean emitting_keystroke;
+  guint code_points_to_delete;
 
   guint keyval;
   guint keycode;
@@ -94,6 +88,7 @@ struct _IBusKeymanEngine {
   IBusLookupTable *table;
   IBusProperty    *status_prop;
   IBusPropList    *prop_list;
+  void            *settings;
 
   commit_queue_item commit_queue[MAX_QUEUE_SIZE];
   commit_queue_item *commit_item;
@@ -219,9 +214,9 @@ static gchar *
 get_context_debug(IBusEngine *engine) {
   IBusKeymanEngine *keyman       = (IBusKeymanEngine *)engine;
 
-  km_core_cp *buf = km_core_state_context_debug(keyman->state, KM_CORE_DEBUG_CONTEXT_CACHED);
+  km_core_cu *buf = km_core_state_context_debug(keyman->state, KM_CORE_DEBUG_CONTEXT_CACHED);
   gchar *result = g_utf16_to_utf8((gunichar2 *)buf, -1, NULL, NULL, NULL);
-  km_core_cp_dispose(buf);
+  km_core_cu_dispose(buf);
   if(result) {
     return result;
   }
@@ -251,17 +246,6 @@ debug_utf8_with_codepoints(const gchar *utf8) {
 #endif
 }
 
-static gboolean
-client_supports_prefilter(IBusEngine *engine)
-{
-  g_assert(engine != NULL);
-#ifdef IBUS_HAS_PREFILTER
-  return (engine->client_capabilities & IBUS_CAP_PREFILTER) != 0;
-#else
-  return FALSE;
-#endif
-}
-
 static gboolean
 client_supports_surrounding_text(IBusEngine *engine) {
   g_assert(engine != NULL);
@@ -292,7 +276,7 @@ set_context_if_needed(IBusEngine *engine) {
   g_message("%s: new application context: |%s| (len:%u) cursor:%d anchor:%d", __FUNCTION__,
     application_context_utf8, context_end - context_start, cursor_pos, anchor_pos);
 
-  km_core_cp *application_context_utf16 = g_utf8_to_utf16(application_context_utf8, -1, NULL, NULL, NULL);
+  km_core_cu *application_context_utf16 = g_utf8_to_utf16(application_context_utf8, -1, NULL, NULL, NULL);
   km_core_context_status result;
   result = km_core_state_context_set_if_needed(keyman->state, application_context_utf16);
   g_free(application_context_utf16);
@@ -324,9 +308,10 @@ ibus_keyman_engine_init(IBusKeymanEngine *keyman) {
   ibus_prop_list_append(keyman->prop_list, keyman->status_prop);
 
   keyman->state = NULL;
+  initialize_keyman_system_service_client();
 }
 
-static km_core_cp* get_base_layout()
+static km_core_cu* get_base_layout()
 {
   return u"en-US";
 
@@ -354,7 +339,7 @@ static km_core_cp* get_base_layout()
     lang = strdup("en-US");
   }
   g_message("lang is %s", lang);
-  km_core_cp *cp = g_utf8_to_utf16(lang, -1, NULL, NULL, NULL);
+  km_core_cu *cp = g_utf8_to_utf16(lang, -1, NULL, NULL, NULL);
   return cp;
   // g_free(lang);
 #endif
@@ -366,8 +351,8 @@ setup_environment(IBusKeymanEngine *keyman)
   g_assert(keyman);
   g_message("%s: setting up environment", __FUNCTION__);
 
-  // Allocate enough options for: 3 environments plus 1 pad struct of 0's
-  km_core_option_item environment_opts[4] = {0};
+  // Allocate enough options for: 4 environments plus 1 pad struct of 0's
+  km_core_option_item environment_opts[5] = {0};
 
   environment_opts[0].scope = KM_CORE_OPT_ENVIRONMENT;
   environment_opts[0].key   = KM_CORE_KMX_ENV_PLATFORM;
@@ -381,6 +366,9 @@ setup_environment(IBusKeymanEngine *keyman)
   environment_opts[2].key   = KM_CORE_KMX_ENV_BASELAYOUTALT;
   environment_opts[2].value = get_base_layout();  // TODO: free when mnemonic layouts are to be supported
 
+  environment_opts[3].scope = KM_CORE_OPT_ENVIRONMENT;
+  environment_opts[3].key   = KM_CORE_KMX_ENV_SIMULATEALTGR;
+  environment_opts[3].value = keyman_get_option_fromdconf(KEYMAN_DCONF_OPTIONS_SIMULATEALTGR) ? u"1" : u"0";
 
   km_core_status status = km_core_state_create(keyman->keyboard, environment_opts, &(keyman->state));
   if (status != KM_CORE_STATUS_OK) {
@@ -389,14 +377,25 @@ setup_environment(IBusKeymanEngine *keyman)
   return status;
 }
 
+static void
+on_dconf_settings_change(GSettings *settings, gchar *key, gpointer user_data) {
+  if (!user_data || g_strcmp0(key, KEYMAN_DCONF_OPTIONS_SIMULATEALTGR) != 0) {
+    // not the SimulateAltGr option...
+    return;
+  }
+  IBusKeymanEngine *keyman = (IBusKeymanEngine *)user_data;
+  km_core_state_dispose(keyman->state);
+  setup_environment(keyman);
+}
+
 void
 free_km_core_option_item(gpointer data) {
   if (!data)
     return;
 
   km_core_option_item *opt = (km_core_option_item *)data;
-  g_free((km_core_cp *)opt->key);
-  g_free((km_core_cp *)opt->value);
+  g_free((km_core_cu *)opt->key);
+  g_free((km_core_cu *)opt->value);
   g_free(opt);
 }
 
@@ -407,7 +406,7 @@ load_keyboard_options(IBusKeymanEngine *keyman) {
   // Retrieve keyboard options from DConf
   // TODO: May need unique packageID and keyboard ID
   g_message("%s: Loading options for kb_name: %s", __FUNCTION__, keyman->kb_name);
-  GQueue *queue_options = keyman_get_options_queue_fromdconf(keyman->kb_name, keyman->kb_name);
+  GQueue *queue_options = keyman_get_keyboard_options_queue_fromdconf(keyman->kb_name, keyman->kb_name);
   int num_options       = g_queue_get_length(queue_options);
   if (num_options < 1) {
     g_queue_free_full(queue_options, free_km_core_option_item);
@@ -483,10 +482,10 @@ ibus_keyman_engine_constructor(
 
     g_strfreev(split_name);
 
-    g_autofree gchar *kmx_file = g_path_get_basename(abs_kmx_path);
-    p = rindex(kmx_file, '.'); // get id to use as dbus service name
+    g_autofree gchar *kmx_filename = g_path_get_basename(abs_kmx_path);
+    p = rindex(kmx_filename, '.'); // get id to use as dbus service name
     if (p) {
-        keyman->kb_name = g_strndup(kmx_file, p-kmx_file);
+        keyman->kb_name = g_strndup(kmx_filename, p-kmx_filename);
         p = rindex(abs_kmx_path, '.');
         if (p)
         {
@@ -501,7 +500,42 @@ ibus_keyman_engine_constructor(
 
     km_core_status status;
 
-    status = km_core_keyboard_load(abs_kmx_path, &(keyman->keyboard));
+    FILE* kmx_file = fopen(abs_kmx_path, "rb");
+    if (!kmx_file) {
+      g_warning("%s: problem opening kmx_file %s. (error: %s).", __FUNCTION__, abs_kmx_path, strerror(errno));
+      return NULL;
+    }
+
+    if (fseek(kmx_file, 0, SEEK_END) < 0) {
+      g_warning("%s: problem seeking to end of kmx_file %s (error: %s).", __FUNCTION__, abs_kmx_path, strerror(errno));
+      fclose(kmx_file);
+      return NULL;
+    }
+    long length = ftell(kmx_file);
+    if (length < 0) {
+      g_warning("%s: problem determining length of kmx_file %s (error: %s).", __FUNCTION__, abs_kmx_path, strerror(errno));
+      fclose(kmx_file);
+      return NULL;
+    }
+    rewind(kmx_file);
+    void* buffer = malloc(length);
+    if (!buffer) {
+      g_warning("%s: problem allocating buffer for reading kmx_file %s (error: %s).", __FUNCTION__, abs_kmx_path, strerror(errno));
+      fclose(kmx_file);
+      return NULL;
+    }
+    if (fread(buffer, 1, length, kmx_file) != length) {
+      g_warning("%s: problem reading entire kmx_file %s (error: %s).", __FUNCTION__, abs_kmx_path, strerror(errno));
+      fclose(kmx_file);
+      free(buffer);
+      return NULL;
+    }
+    fclose(kmx_file);
+
+    status = km_core_keyboard_load_from_blob(abs_kmx_path, buffer, length,
+      &(keyman->keyboard));
+
+    free(buffer);
 
     if (status != KM_CORE_STATUS_OK) {
       g_warning("%s: problem loading km_core_keyboard %s. Status is %u.", __FUNCTION__, abs_kmx_path, status);
@@ -515,12 +549,16 @@ ibus_keyman_engine_constructor(
       return NULL;
     }
 
+    keyman->settings = keyman_subscribe_option_changes(on_dconf_settings_change, keyman);
+
     status = load_keyboard_options(keyman);
     if (status != KM_CORE_STATUS_OK) {
       ibus_keyman_engine_destroy(keyman);
       return NULL;
     }
 
+    ping_keyman_system_service();
+
     set_context_if_needed(engine);
 
     return (GObject *) keyman;
@@ -537,6 +575,11 @@ ibus_keyman_engine_destroy (IBusKeymanEngine *keyman)
     g_assert (engine_name);
     g_message("DAR: %s %s", __FUNCTION__, engine_name);
 
+    if (keyman->settings) {
+      keyman_unsubscribe_option_changes(keyman->settings, on_dconf_settings_change, keyman);
+      keyman->settings = NULL;
+    }
+
     if (keyman->prop_list) {
         g_debug("DAR: %s: unref keyman->prop_list", __FUNCTION__);
         g_object_unref (keyman->prop_list);
@@ -602,17 +645,18 @@ process_output_action(IBusEngine *engine, const km_core_usv* output_utf32) {
   IBusKeymanEngine *keyman = (IBusKeymanEngine *)engine;
   gchar *output_utf8 = g_ucs4_to_utf8(output_utf32, -1, NULL, NULL, NULL);
   g_autofree gchar *debug  = NULL;
-  if (client_supports_prefilter(engine) && !client_supports_surrounding_text(engine)) {
-    // non-compliant app with patched ibus
-    g_message("%s: Adding to commit queue: %s", __FUNCTION__, debug = debug_utf8_with_codepoints(output_utf8));
+
+  if (client_supports_surrounding_text(engine)) {
+    // compliant app
+    g_message("%s: compliant app: Outputing %s", __FUNCTION__, debug = debug_utf8_with_codepoints(output_utf8));
+    commit_string(keyman, output_utf8);
+    g_free(output_utf8);
+  } else {
+    // non-compliant app
+    g_message("%s: non-compliant app: Adding to commit queue: %s", __FUNCTION__, debug = debug_utf8_with_codepoints(output_utf8));
     g_assert(keyman->commit_item->char_buffer == NULL);
     keyman->commit_item->char_buffer = output_utf8;
     // don't free output_utf8 - assigned to char_buffer!
-  } else {
-    // compliant app or unpatched ibus
-    g_message("%s: Outputing %s", __FUNCTION__, debug = debug_utf8_with_codepoints(output_utf8));
-    commit_string(keyman, output_utf8);
-    g_free(output_utf8);
   }
 }
 
@@ -637,11 +681,9 @@ process_backspace_action(IBusEngine *engine, unsigned int code_points_to_delete)
     g_message("%s: compliant app: deleting surrounding text %d codepoints", __FUNCTION__, code_points_to_delete);
     ibus_engine_delete_surrounding_text(engine, -code_points_to_delete, code_points_to_delete);
   } else {
-    g_message("%s: non-compliant app: forwarding %d backspaces", __FUNCTION__, code_points_to_delete);
-    while (code_points_to_delete > 0) {
-      ibus_engine_forward_key_event(engine, KEYMAN_BACKSPACE_KEYSYM, KEYMAN_BACKSPACE, 0);
-      code_points_to_delete--;
-    }
+    g_message("%s: non-compliant app: queueing %d backspaces", __FUNCTION__, code_points_to_delete);
+    IBusKeymanEngine *keyman = (IBusKeymanEngine *)engine;
+    keyman->commit_item->code_points_to_delete = code_points_to_delete;
   }
 }
 
@@ -655,7 +697,7 @@ process_persist_action(IBusEngine *engine, km_core_option_item *persist_options)
     g_assert(option->key != NULL && option->value != NULL);
     g_message("%s: Saving keyboard option to DConf", __FUNCTION__);
     // Load the current keyboard options from DConf
-    keyman_put_options_todconf(keyman->kb_name, keyman->kb_name, (gchar *)option->key, (gchar *)option->value);
+    keyman_put_keyboard_options_todconf(keyman->kb_name, keyman->kb_name, (gchar *)option->key, (gchar *)option->value);
   }
 }
 
@@ -665,8 +707,8 @@ process_emit_keystroke_action(IBusEngine *engine, km_core_bool emit_keystroke) {
     return;
   }
   IBusKeymanEngine *keyman = (IBusKeymanEngine *)engine;
-  if (!client_supports_prefilter(engine) || client_supports_surrounding_text(engine)) {
-    // compliant app or unpatched ibus version
+  if (client_supports_surrounding_text(engine)) {
+    // compliant app
     ibus_engine_forward_key_event(engine, keyman->commit_item->keyval,
       keyman->commit_item->keycode, keyman->commit_item->state);
     return;
@@ -687,21 +729,42 @@ process_capslock_action(km_core_caps_state caps_state) {
 
 static void
 commit_current_queue_item(IBusKeymanEngine *keyman) {
-  // only called for non-compliant apps with patched ibus
+  // only called for non-compliant apps
   g_assert(keyman != NULL);
-  g_assert(client_supports_prefilter((IBusEngine *)keyman));
-  g_assert(!client_supports_surrounding_text((IBusEngine *)keyman));
+  IBusEngine* engine = (IBusEngine *)keyman;
+  g_assert(!client_supports_surrounding_text(engine));
 
-  if (keyman->commit_item <= keyman->commit_queue)
+  if (keyman->commit_item <= keyman->commit_queue){
+    g_message("%s: queue is empty", __FUNCTION__);
     return;
+  }
 
   commit_queue_item *current_item = &keyman->commit_queue[0];
+  if (current_item->code_points_to_delete > 0) {
+    g_message("%s: Forwarding %d backspaces from commit queue", __FUNCTION__, current_item->code_points_to_delete);
+    while (current_item->code_points_to_delete > 0) {
+      ibus_engine_forward_key_event(engine, KEYMAN_BACKSPACE_KEYSYM, KEYMAN_BACKSPACE, 0);
+      current_item->code_points_to_delete--;
+    }
+    // don't remove the item from the queue yet - we need to process it
+    // again for the output and keystrokes. Instead emit the sentinel key
+    // again.
+    g_message("%s: Forcing ordered output", __FUNCTION__);
+    call_ordered_output_sentinel();
+    return;
+  }
   if (current_item->char_buffer != NULL) {
+    g_autofree gchar *debug = NULL;
+    g_message("%s: Committing from commit queue: %s", __FUNCTION__,
+      debug = debug_utf8_with_codepoints(current_item->char_buffer));
     commit_string(keyman, current_item->char_buffer);
     g_free(current_item->char_buffer);
+    current_item->char_buffer = NULL;
   }
   if (current_item->emitting_keystroke) {
-    ibus_engine_forward_key_event((IBusEngine*)keyman, current_item->keyval, current_item->keycode, current_item->state);
+    g_message("%s: Forwarding key from commit queue: keyval=0x%02x, keycode=0x%02x, state=0x%02x",
+      __FUNCTION__, current_item->keyval, current_item->keycode, current_item->state);
+    ibus_engine_forward_key_event(engine, current_item->keyval, current_item->keycode, current_item->state);
   }
   keyman->commit_item--;
   memmove(keyman->commit_queue, &keyman->commit_queue[1], sizeof(commit_queue_item) * MAX_QUEUE_SIZE - 1);
@@ -712,30 +775,41 @@ static void
 finish_process_actions(IBusEngine *engine) {
   g_assert(engine != NULL);
   IBusKeymanEngine *keyman = (IBusKeymanEngine *)engine;
-  if (!client_supports_prefilter(engine) || client_supports_surrounding_text(engine)) {
-    // compliant app or unpatched ibus
+  if (client_supports_surrounding_text(engine)) {
+    // compliant app
     return;
   }
 
-  // non-compliant app with patched ibus
-  guint state = keyman->commit_item->state;
-  keyman->commit_item++;
-  if (keyman->commit_item > &keyman->commit_queue[MAX_QUEUE_SIZE-1]) {
-    g_error("Overflow of keyman commit_queue!");
-    // TODO: log to Sentry
-    keyman->commit_item--;
-  }
+  // non-compliant app
+  guint keycode = keyman->commit_item->keycode;
 
-  // Forward a fake key event to get the correct order of events so that any backspace key we
-  // generated will be processed before the character we're adding. We need to send a
-  // valid keyval/keycode combination so that it doesn't get swallowed by GTK but which
-  // isn't very likely used in real keyboards. F24 seems to work for that.
-  ibus_engine_forward_key_event(engine,
-    KEYMAN_NOCHAR_KEYSYM,
-    KEYMAN_F24_KEYCODE_OUTPUT_SENTINEL,
-    (state & IBUS_RELEASE_MASK)
-      ? IBUS_PREFILTER_MASK | IBUS_RELEASE_MASK
-      : IBUS_PREFILTER_MASK);
+  switch (keycode) {
+    case KEYMAN_LSHIFT:
+    case KEYMAN_RSHIFT:
+    case KEYMAN_LCTRL:
+    case KEYMAN_RCTRL:
+    case KEYMAN_LALT:
+    case KEYMAN_RALT:
+      // we don't forward modifier keys that the user holds while pressing another
+      // key.
+      g_message("%s: Ignoring modifier key", __FUNCTION__);
+      break;
+    default:
+      keyman->commit_item++;
+      if (keyman->commit_item > &keyman->commit_queue[MAX_QUEUE_SIZE - 1]) {
+        g_error("Overflow of keyman commit_queue!");
+        // TODO: log to Sentry
+        keyman->commit_item--;
+      }
+
+      // Forward a fake key event to get the correct order of events so that any backspace key we
+      // generated will be processed before the character we're adding. We need to send a
+      // valid keycode so that it doesn't get swallowed by GTK but which isn't very likely used
+      // in real keyboards. F24 seems to work for that.
+      g_message("%s: Forcing ordered output", __FUNCTION__);
+      call_ordered_output_sentinel();
+      break;
+  }
 }
 
 static void
@@ -768,13 +842,26 @@ ibus_keyman_engine_process_key_event(
 
   g_message("-----------------------------------------------------------------------------------------------------------------");
   g_message(
-      "DAR: %s - keyval=0x%02x keycode=0x%02x, state=0x%02x, isKeyDown=%d, supports_prefilter=%d, compliant=%d", __FUNCTION__, keyval, keycode,
-      state, isKeyDown, client_supports_prefilter(engine), client_supports_surrounding_text(engine));
+      "DAR: %s - keyval=0x%02x keycode=0x%02x, state=0x%02x, isKeyDown=%d, compliant=%d", __FUNCTION__, keyval, keycode,
+      state, isKeyDown, client_supports_surrounding_text(engine));
 
   // This keycode is a fake keycode that we send when it's time to commit the text, ensuring the
   // correct output order of backspace and text.
-  if (client_supports_prefilter(engine) && !client_supports_surrounding_text(engine) &&
-      keycode == KEYMAN_F24_KEYCODE_OUTPUT_SENTINEL && (state & IBUS_PREFILTER_MASK)) {
+  if (!client_supports_surrounding_text(engine) && keycode == KEYMAN_F24_KEYCODE_OUTPUT_SENTINEL) {
+    // non-compliant app
+    if (!isKeyDown) {
+      g_message("%s: got F24 Sentinel, ignore keyup", __FUNCTION__);
+      return TRUE;
+    }
+    g_message("%s: got F24 Sentinel, queue content:", __FUNCTION__);
+    for (int i = 0; i < MAX_QUEUE_SIZE && &keyman->commit_queue[i] != keyman->commit_item; i++) {
+      commit_queue_item *item = &keyman->commit_queue[i];
+      g_message(
+          "    queue item %d:  keyval=0x%02x keycode=0x%02x, state=0x%02x, char_buffer=%s, "
+          "emitting_keystroke=%d, code_points_to_delete=%d",
+          i, item->keyval, item->keycode, item->state, item->char_buffer,
+          item->emitting_keystroke, item->code_points_to_delete);
+    }
     commit_current_queue_item(keyman);
     return TRUE;
   }
@@ -811,7 +898,7 @@ ibus_keyman_engine_process_key_event(
   // #10476: Core currently doesn't handle IBUS_MOD{2-4}_MASK modifiers.
   // On Ubuntu 23.10/24.04 we get IBUS_MOD4_MASK set on keycode 0x39 (space) when
   // the user tries to switch keyboards. Since this is not a regular keypress
-  // and Core doesn't handle it, we need to just return and let the Gnome deal
+  // and Core doesn't handle it, we need to just return and let GTK deal
   // with it. We could consider to add it to Core and let Core ignore it.
   // As for IBUS_MOD3_MASK it's unclear when/how that gets set, so we
   // just not deal with that for now until we notice problems.
@@ -825,7 +912,7 @@ ibus_keyman_engine_process_key_event(
   // it. At the moment however we let Core process the keypress and since
   // it doesn't have rules for the numeric keypad keys we eventually
   // forward the key to ibus (in process_emit_keystroke_action) and let
-  // Gnome deal with it.
+  // GTK deal with it.
 
   // keyman modifiers are different from X11/ibus
   uint16_t km_mod_state = 0;
@@ -871,15 +958,14 @@ ibus_keyman_engine_process_key_event(
 
   process_actions(engine, core_actions);
 
-  // If we have a new ibus version that supports prefilter and a non-compliant
-  // client, i.e. a client that doesn't support surrounding text (e.g.
-  // Chromium as of v104) we forwarded the key event with IBUS_PREFILTER_MASK
-  // set and now stop further processing by returning TRUE.
-  // With an old ibus version without prefilter support as well as with
-  // a compliant client (i.e. it does support surrounding text), we return
+  // If we have a non-compliant client, i.e. a client that doesn't support
+  // surrounding text (e.g. Chromium as of v104) we sent the key event
+  // to the system service and now stop further processing by returning TRUE.
+  // With a compliant client (i.e. it does support surrounding text), we return
   // TRUE because we completely processed the event and no further
   // processing should happen.
   g_message("%s: after processing all actions: %s", __FUNCTION__, debug_context2 = get_context_debug(engine));
+
   return TRUE;
 }
 
